VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsUnPackage"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' UnPackage Class v1.0.
'
' Designed and coded by Leigh Bowers.
' Copyright 1999 Compulsion Software Development
'
' WWW:   http://www.esheep.freeserve.co.uk/compulsion
' Email: compulsion@esheep.freeserve.co.uk
'
' Note:
' Error handling is very basic in this first release -
' Methods simply return True (Success) or False (Fail).
' No compression (implement in v2.0 ???).
' Package files do not have to have the .pac extension.
'
' Use the FilePackager application to create packages.
' See the example TestClass source for info on how to
' use the UnPackage methods\properties.
'
' This class may be used freely for personal use ONLY.

Private bNumberOfFiles As Byte  ' Max 255
Private sPackage As String      ' Package Name
Private sDestination As String  ' Destination Dir
Private iBuffer As Integer      ' Buffer Size (Max 32k)

Private Type FileInfo           ' File Details
    Name As String
    Length As Long
    StartPos As Long
End Type

' Array to hold list of File Details (above)

Private Files() As FileInfo


Public Function ExtractFile(sFilename As String) As Boolean

    ' This method extracts the specified file from the
    ' package and places it in the destination
    ' directory (.Destination property).
    
    ' If a package hasn't previously been Initialised
    ' (using the .Initialise method), it will be auto
    ' initialised by this routine prior to extraction.
    
    ' If the .Buffer property hasn't been set, it is
    ' deaulted to 2k (2048 bytes).

    Dim bLoop As Byte
    Dim iFileNumber As Integer
    Dim iInputFile As Integer
    Dim iOutputFile As Integer
    Dim iFileBuffer As Integer
    Dim lDone As Long
    Dim sBuffer As String
    
     On Error GoTo Error_Handler
     
     ' If package hasn't been initialised, init now
     If bNumberOfFiles = 0 Then Initialise
        
    ' Ensure file exists in the package
    iFileNumber = -1
    For bLoop = 0 To bNumberOfFiles - 1
        If UCase(Files(bLoop).Name) = UCase(sFilename) Then
            iFileNumber = bLoop
        End If
    Next ' bLoop
    If iFileNumber = -1 Then GoTo Error_Handler
    
    ' If no buffer size has been set, use default of 2k
    iFileBuffer = IIf(iBuffer = 0, 2048, iBuffer)
    
    ' Done (bytes already done) = ZERO
    lDone = 0
    
    ' Open input and output files
    iInputFile = FreeFile
    Open sPackage For Binary As iInputFile
    iOutputFile = FreeFile
    Open sDestination + sFilename For Binary As iOutputFile
    
    ' UnPackage the specified file to the destination.
    ' Uses the Buffer property.
    
    If iFileBuffer > Files(iFileNumber).Length Then
        iFileBuffer = Files(iFileNumber).Length
    End If
    sBuffer = String(iFileBuffer, " ")
    
    Get iInputFile, Files(iFileNumber).StartPos, sBuffer
    Put iOutputFile, , sBuffer
    lDone = iFileBuffer
    
    Do Until lDone = Files(iFileNumber).Length
        sBuffer = String(iFileBuffer, " ")
        Get iInputFile, , sBuffer
        Put iOutputFile, , sBuffer
        lDone = lDone + iFileBuffer
        If iFileBuffer > (Files(iFileNumber).Length - lDone) Then
            iFileBuffer = (Files(iFileNumber).Length - lDone)
        End If
    Loop
    
    ' Finished!
    ExtractFile = True
    
Error_Handler:

    Close iOutputFile
    Close iInputFile

End Function




Public Function Initialise() As Boolean

    ' This method retrieves header information from
    ' the package. It takes all file details
    ' (Name, Size and Start Position) and populates
    ' the Files() array.

    Dim iFreeFile As Integer
    Dim sInput As String
    Dim bLoop As Byte
    Dim bLenFileName() As Byte

    On Error GoTo Error_Handler
    
    ' Ensure package exists
    If Dir(sPackage) = "" Then GoTo Error_Handler

    ' Clear down the array of files ready for
    ' re-population.
    ReDim Files(0)

    ' Open the package and retrieve header information
    iFreeFile = FreeFile
    Open sPackage For Binary Access Read As iFreeFile
    
    ' Check for valid identifier "LBFPvvv" (vvv=Version)
    sInput = String(7, " ")
    Get iFreeFile, , sInput
    If Left(sInput, 4) <> "LBFP" Then GoTo Error_Handler
    
    ' Retrieve number of file held in package
    Get iFreeFile, , bNumberOfFiles
    
    ' Retrieve filename lengths and file lengths
    For bLoop = 0 To bNumberOfFiles - 1
        ReDim Preserve bLenFileName(bLoop)
        Get iFreeFile, , bLenFileName(bLoop)
        ReDim Preserve Files(bLoop)
        Get iFreeFile, , Files(bLoop).Length
    Next ' bLoop
    
    ' Retrieve filenames
    For bLoop = 0 To bNumberOfFiles - 1
        Files(bLoop).Name = String(bLenFileName(bLoop), " ")
        Get iFreeFile, , Files(bLoop).Name
    Next ' bLoop
    
    ' Calculate file start positions within the package
    Files(0).StartPos = Seek(iFreeFile)
    For bLoop = 1 To bNumberOfFiles - 1
        Files(bLoop).StartPos = Files(bLoop - 1).StartPos + Files(bLoop - 1).Length
    Next ' bLoop
    
    ' Package initialised
    Initialise = True
    
Error_Handler:
    
    Close iFreeFile
    
End Function
Public Property Let Package(sPassPackage As String)

    ' Specify the package to be used.
    ' Multiple packages can be used at any one time by
    ' creating multiple instances of this class.
    
    ' You must include the full pathname.

    sPackage = sPassPackage

End Property


Public Property Let Destination(sPassDestination As String)

    ' Specify the destination directory

    sDestination = RTrim(sPassDestination)
    
    If Right(sDestination, 1) <> "\" Then sDestination = sDestination + "\"

End Property

Public Property Let Buffer(iPassBuffer As String)

    ' Specify buffer size in BYTES

    iBuffer = iPassBuffer

End Property

Public Function Remove(sFilename As String) As Boolean

    ' This method removes a previously extracted file
    ' from the destination directory.
    ' Note: This routine doesn't remove the file from
    '       the package.

    On Error GoTo Error_Handler
    
    ' Remove unpackaged file from destination
    
    Kill sDestination + sFilename
    
    ' File removed!
    
    Remove = True
    
Error_Handler:

End Function

Public Property Get NumberOfFiles() As Integer
    
    ' Used to determine how many files are contained
    ' within the current package.

    NumberOfFiles = bNumberOfFiles

End Property

Public Property Get GetFilename(bFileNumber As Byte) As String

    ' Retrieve filename(n).
    ' Use the .NumberOfFiles property to determine
    ' the number of files contained within the package.

    If bFileNumber <= bNumberOfFiles Then
        GetFilename = Files(bFileNumber).Name
    Else
        GetFilename = vbNull
    End If

End Property


